import pyresdev.utils._developer_utils as devutils
import pyresdev.utils as utils

# General
import datetime
import os
import pickle
import warnings
import argparse
import logging
import json
from datetime import date
from math import log
# Data
import pandas as pd
import numpy as np
import regex as re
# ML
# Used in : Data_Engineering
from sklearn.experimental import enable_iterative_imputer
from sklearn.impute import IterativeImputer  # Used in : Data_Engineering
from sklearn.linear_model import BayesianRidge  # Used in : Data_Engineering
from sklearn.exceptions import DataConversionWarning

__all__ = ['preprocessing_training']


def preprocessing_training_common(df, config, industry_dictionary, state_dictionary,
                                  mx_prob_dictionary, mx_bv_dictionary, model_type):
    """Common functions used for every campaign in the processing

    Args:
        df ([DataFrame]): [Dataframe to split]
        config ([Dict]): [Config for the campaign]
        Industry_dictionary ([dict]): [Industry dictionary]
        mx_prob_dictionary(dict) : MX dictionary converting to lead performance
        mx_bv_dictionary(dict) : MX Dictionary converting to bv performance
        state_dictionary(dict): state dictionary converting to bv performance
        model_type(dict): model used ( bv, probabiltiy or multiclass)
    Returns:
        [DataFrames]: [Training and testing divided into dependent and independent variables ]
    """
    # Set Types
    df = devutils.set_dataframe_columntypes( df )

    # Delete some statuses in training cases and create target column, set index in prediction cases
    status_to_delete = config['status_to_delete']
    df = devutils.clean_previous_status( df=df, status_to_clean=status_to_delete )

    # Eliminate rows with NA in the following columns ( the have a few NA)
    cols_to_clean_na = config['cols_to_clean_na']
    df = devutils.clean_rows_with_na( df=df, cols_to_clean_na=cols_to_clean_na )

    # Map industry dictionary
    df = df.replace( {'Industry': industry_dictionary} )
    logging.info( df['Industry'].value_counts() )

    # Fill NA with "Uknown" in the  columns specified in the config
    cols_to_fill = config['fill_unknown']
    df = devutils.fill_na_with_unknown( df=df, columns=cols_to_fill )

    df = devutils.clean_mxtype_using_dictionary( df=df, mx_bv_dict=mx_bv_dictionary,
                                                 mx_prob_dict=mx_prob_dictionary )

    # Map State dictionary
    df = df.replace( {'State': state_dictionary} )
    logging.info( df['State'].value_counts() )

    # add the month of the transaction
    df = devutils.create_month_status( df=df )

    # Create Company years column
    df = devutils.create_company_years_column( df=df )

    # Create new seniorities and the rest go to "other"
    df = devutils.fill_seniority_column( df=df )

    # Take the countries of the campaign from the config file
    countries_list = config['countries_list']
    df = devutils.filter_countries( df=df, countries=countries_list )

    # Fix errors in CompanyFollowers
    df = devutils.clean_companyfollowers_text( df=df )

    # Converting kw_key and technologies list of words to an int, using regex
    df = devutils.convert_kwkey_specialties_and_technologies_to_int( df=df )

    # Historic variables, fill nulls with 0
    df = devutils.fill_historic_variables( df=df )

    # Use imputer
    impute_columns = config['impute_columns']
    df_leads_clean, imp = devutils.impute_nas_and_save_imputer( df=df, columns_to_impute=impute_columns )
    imputer_name = config['imputer_name']
    imputer_output_path = os.path.join( '/opt/ml/processing/imputer', imputer_name )
    logging.info( 'Saving iterative imputer to {}'.format( imputer_output_path ) )
    pickle.dump( imp, open( imputer_output_path, 'wb' ) )

    # Logs
    cols_to_transform = config['cols_to_transform']
    df = devutils.log_transformation( df_leads_clean, columns=cols_to_transform )

    # Dropping unused variables
    print( model_type )
    cols_to_drop = config['cols_to_drop'][model_type]
    df = devutils.drop_columns( df=df, columns=cols_to_drop )

    return df


def preprocessing_training(df, industry_dictionary, state_dictionary,
                           mx_bv_dictionary, mx_prob_dictionary, config, model_type):
    """[Pre-processing function to prepare the data for training the model and to make predictions]

    Args:
        df ([Pandas Dataframe]): [Pandas Dataframe with the raw data ]
        industry_dictionary ([Dict]): [Dictionary to reduce the cardinality of the "Industry attribute"]
        config(dict) : Dictionary with the configuration ( read from json)
        model_type(string) : type of the model to be used
        mx_bv_dictionary(string): Dictionary of MX converting to bv performance
        mx_prob_dictionary(string): Dictionary of MX converting to prob performance
        model_type(string) : can be probability, bv or multiclass
    Returns:
        df_clean[Pandas Dataframe]: Pre-processed DataFrame ready for training purposes
    """

    # Make a copy of the dataset
    df_leads_clean = df.copy()

    nna = df_leads_clean.isna().sum()
    logging.info( f"NA in original dataframe:" )
    logging.info( f"{nna}" )

    df_leads_clean = preprocessing_training_common( df=df_leads_clean,
                                                    config=config,
                                                    industry_dictionary=industry_dictionary,
                                                    mx_prob_dictionary=mx_prob_dictionary,
                                                    mx_bv_dictionary=mx_bv_dictionary,
                                                    state_dictionary=state_dictionary,
                                                    model_type=model_type )

    df_leads_clean = utils.reduce_mem_usage_category( df=df_leads_clean )
    df_leads_clean = utils.onehotencoding( df=df_leads_clean )

    # Prepare target column
    df_leads_clean = devutils.set_target_column(df=df_leads_clean,model_type=model_type,bins=config['bins'])

    nna = df_leads_clean.isna().sum()
    logging.info( f"NA in final dataframe:" )
    logging.info( f"{nna}" )

    return df_leads_clean



# AWSTools

from ._aws_tools import *
from ._memory_reduction import *
from ._processing_tools import *

__all__ = (_aws_tools.__all__ +
           _memory_reduction.__all__+
           _processing_tools.__all__)




import logging
import json
import pickle
import os
import pandas as pd


__all__=['one_hot_encoding','read_config_file','drop_columns','import_dictionary']


def import_dictionary (path,file):
    with open( os.path.join( path, file ), 'rb' ) as f:
        logging.info( f'reading input dict from {path}' )
        dict = pickle.load( f )
        return dict


def drop_columns(df, columns):
    """Drops the selected columns

    Args:
        df ([dataframe]): [leads dataframe]
        columns ([list]): [list columns to drop]

    Returns:
        [df]: [leads dataframe]
    """
    for column in columns:
        if column in df:
            logging.info(f'dropping column : {column}')
            df = df.drop(column, axis=1)
    return df


def one_hot_encoding(df):
    """Generate one hot encoded dataset
    WARNING:
        Only encodes colums with category

    Args:
        df ([dataframe]): [dataframe]

    Returns:
        df[dataframe]: [dataframe with categorical variables one hot encoded]
    """
    # These variables must not be categories
    if 'campaign_bulk_date' in df.columns:
        df['campaign_bulk_date'] = df['campaign_bulk_date'].astype('object')
    if 'IDEmailStatus' in df.columns:
        df['IDEmailStatus'] = df['IDEmailStatus'].astype('object')
    logging.info("Data Types before encoding:")
    logging.info(f"{df.dtypes}")
    df = pd.get_dummies(data=df,
                        columns=df.select_dtypes(include=['category']).columns.tolist())
    logging.info("Data Types after encoding:")
    logging.info(f"{df.dtypes}")
    # Convert date
    if 'campaign_bulk_date' in df.columns:
        df['campaign_bulk_date'] = df['campaign_bulk_date'].astype('object')
        df['campaign_bulk_date'] = pd.to_datetime(df['campaign_bulk_date'])
        df['campaign_bulk_date'] = df['campaign_bulk_date'].dt.date
    else:
        raise Exception('campaign_bulk_date not present! unable to perform dataset split')
    return df


def read_config_file(path, campaign_id):
    try:
        logging.info(f'reading config from {path}')
        with open( path ) as file:
            data = file.read()
        data = dict(json.loads( data ))
        logging.info(f'accessing {campaign_id}')
        config = data[campaign_id]

    except Exception as exception:
        config = None
        error_message = f"Read config file - {repr( exception )}"
        logging.info( error_message )
    return config





def set_dataframe_columntypes(df):
    """Manual setting of the datatypes

    Args:
        df ([dataframe]): [dataframe with the leads information]

    Returns:
       df [df]: [Dataframe with the leads information, with the new types]
    """
    logging.info("Changing data types")
    df = clean_rows_with_na( df=df, cols_to_clean_na=['IDEmailStatus'] )
    if 'idLead' in df:
        df['idLead'] = df['idLead'].astype('int32')
    if 'idCampaign' in df:
        df['idCampaign'] = df['idCampaign'].astype('int8')
    if 'campaign_bulk_date' in df:
        df['campaign_bulk_date'] = df['campaign_bulk_date'].astype('object')
        df['campaign_bulk_date'] = pd.to_datetime(df['campaign_bulk_date'])
        df['campaign_bulk_date'] = df['campaign_bulk_date'].dt.date
    if 'IDEmailStatus' in df:
        df['IDEmailStatus'] = df['IDEmailStatus'].astype('int8')
    if 'Ranking' in df:
        df['Ranking'] = df['Ranking'].astype('float32')
    if 'BV' in df:
        df['BV'] = df['BV'].astype('float32')
    if 'Size' in df:
        df['Size'] = df['Size'].astype('object')
    if 'Industry' in df:
        df['Industry'] = df['Industry'].astype('object')
    if 'CompanyFollowers' in df:
        df['CompanyFollowers'] = df['CompanyFollowers'].astype('object')
    if 'QEmployeesOnLinkedIn' in df:
        df['QEmployeesOnLinkedIn'] = df['QEmployeesOnLinkedIn'].astype('float64')
    if 'Type' in df:
        df['Type'] = df['Type'].astype('object')
    if 'Specialties' in df:
        df['Specialties'] = df['Specialties'].astype('object')
    if 'MXtype' in df:
        df['MXtype'] = df['MXtype'].astype('object')
    if 'Technologies' in df:
        df['Technologies'] = df['Technologies'].astype('object')
    if 'Country' in df:
        df['Country'] = df['Country'].astype('object')
    if 'State' in df:
        df['State'] = df['State'].astype('object')
    if 'Title' in df:
        df['Title'] = df['Title'].astype('object')
    if 'kw_key' in df:
        df['kw_key'] = df['kw_key'].astype('object')
    if 'Seniority' in df:
        df['Seniority'] = df['Seniority'].astype('object')
    if 'Department' in df:
        df['Department'] = df['Department'].astype('object')
    if 'LeadConnections' in df:
        df['LeadConnections'] = df['LeadConnections'].astype('float64')
    if 'LeadFollowers' in df:
        df['LeadFollowers'] = df['LeadFollowers'].astype('float64')
    if 'PreviousContacts' in df:
        df['PreviousContacts'] = df['PreviousContacts'].astype('float16')
    if 'PreviousStatus' in df:
        df['PreviousStatus'] = df['PreviousStatus'].astype('float16')
    if 'DaysSinceLastContact' in df:
        df['DaysSinceLastContact'] = df['DaysSinceLastContact'].astype(
            'float16')
    if 'DaysSinceFirstContact' in df:
        df['DaysSinceFirstContact'] = df['DaysSinceFirstContact'].astype(
            'float16')
    return df


def clean_previous_status(df, status_to_clean):
    """Eliminate rows with a defined previous status value
    Args:
        df ([dataframe]): [dataframe with the leads]
        status_to_clean ([list of ints]): [list with the status to clean]

    Returns:
        df [dataframe]: [dataframe with the leads]
    """

    logging.info(f"Cleaning invalid previous status ( {status_to_clean})")
    for status in status_to_clean:
        if 'PreviousStatus' in df:
            df = df[df.PreviousStatus != status]
    return df


def clean_rows_with_na(df,cols_to_clean_na):
    """Eliminates rows with NAs in Attributes with low % NA Percentage

    Args:
        df ([dataframe]): Leads dataframe
    Returns:
        df [dataframe]: [dataframe with the leads]
    """
    # Logging  amount of rows
    nrows_pre = df.shape[0]
    logging.info( f"Original rows: {nrows_pre}" )

    for col in cols_to_clean_na:
        if col in df:
            nas_sum = df[col].isna().sum()
            logging.info(f"NAs in {col}: {nas_sum}")
            df = df[df[col].notna()]
    nrows_post = df.shape[0]
    percentage_eliminated = (nrows_pre - nrows_post) / nrows_pre * 100
    logging.info( f" Rows after deleting rows with NA: {nrows_post}, Percentage: {percentage_eliminated}" )

    return df


def fill_na_with_unknown(df, columns):
    """[summary]
    Args:
        df ([dataframe]): [dataframe with the leads]
        columns ([list of strings]): [list with the names of the columns]
    Returns:
        df [dataframe]: [dataframe with the leads]
    """
    for col in columns:
        if col in df.columns:
            logging.info(f'filling NA in {col} with "unknown" ')
            df[col] = df[col].fillna('Unknown')
    return df


def create_specialties_flag(df):
    """Converts specialties to a flag that determines if the company has them

    Args:
        df ([dataframe]): Dataframe with leads
    Returns:
        df ([dataframe]): Dataframe with leads
    """
    logging.info("Creating Specialties flag")
    df['Has Specialties'] = df['Specialties'].isna()
    df['Has Specialties'] *= 1
    df['Has Specialties'] = df['Has Specialties'].astype("category")
    return df


def set_idemailstatus_target_column(df):

    if ('IDEmailStatus' in df.columns):
        logging.info('Setting E-Mail status target ')
        df = df[~df['IDEmailStatus'].isin([3, 5, 7, 32])]
        df.loc[df['IDEmailStatus'] != 14, 'IDEmailStatus'] = 0
        df.loc[df['IDEmailStatus'] == 14, 'IDEmailStatus'] = 1
    else:
        logging.error('No IDEmailSTatus column found')
        raise Exception('No IDEmailStatus column found!!!!!')
    return df


def set_bv_target_column(df):
    if('BV' in df.columns):
        logging.info('Setting BV status target ')
        df = df[df.BV.notna()]
        df['BV']=df['BV'].astype('float32')
    else:
        logging.error('No BV column found')
        raise Exception('No BV column found!!!!!')
    #   df = df[df.BV < 40]
    return df


def set_bv_discrete_target_column(df,bins):
    if('BV' in df.columns):
        logging.info('Setting BV status target for classification ')
        df.BV.fillna(value=0,inplace=True)
        labels = [0, 1, 2, 3, 4, 5]
        df.insert(loc=1,column='BV_cut', value= pd.cut(x=df['BV'], bins = bins,labels=labels).astype('float32'))
    else:
        logging.error('No BV column found')
        raise Exception('No BV column found!!!!!')
    return df

def set_target_column(df,model_type,bins):
    if model_type == 'probability':
        df = set_idemailstatus_target_column( df=df )
    elif model_type == 'bv':
        df = set_bv_target_column( df=df )
    elif model_type == 'multiclass':
        df = set_bv_discrete_target_column( df=df,bins=bins )
    else:
        raise Exception( 'Incorrect model_type !!!!!' )
    return df

def create_company_years_column(df):
    """Year founded, creation of the Company years category using Year founded

    Args:
        df ([dataframe]): [dataframe with the leads]

    Returns:
        df [dataframe]: [dataframe with the leads]
    """

    logging.info("Creating company years category")
    # Create Company years category
    if df['YearFounded'].dtype == object:
        df['YearFounded'] = df['YearFounded'].str.extract(
            '(\d{4})$', expand=False).astype('Float16')

    CurrentYear = date.today().year
    df['YearFounded'] = df['YearFounded'].fillna(0)
    df.loc[(CurrentYear - df['YearFounded'] <= 2),
           'CompanyYears'] = "0-2 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 2) & (
        CurrentYear - df['YearFounded'] <= 5), 'CompanyYears'] = "0-2 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 2) & (
        CurrentYear - df['YearFounded'] <= 5), 'CompanyYears'] = "3-5 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 5) & (
        CurrentYear - df['YearFounded'] <= 10), 'CompanyYears'] = "6-10 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 10) & (
        CurrentYear - df['YearFounded'] <= 25), 'CompanyYears'] = "11-25 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 25) & (
        CurrentYear - df['YearFounded'] <= 50), 'CompanyYears'] = "26-50 Years"
    df.loc[(CurrentYear - df['YearFounded'] > 50),
           'CompanyYears'] = "51+ Years"
    # Errors go to "unknown category"
    df.loc[(df['YearFounded'] > CurrentYear) | (
        df['YearFounded'] < 1600), 'CompanyYears'] = "Unknown"
    df = df.drop(['YearFounded'], axis=1)
    return df


def fill_seniority_column(df):
    """Defines new cases for Seniority and sets the rest to 'other'

    Args:
        df ([dataframe]): [Dataframe with the leads]
    Returns:
        df [datafrane]: [dataframe with the leads]
    """
    logging.info("Filling Seniority from some Titles")
    df.loc[(df['Title'].str.contains('Specialist', 'Title')) &
           (df['Seniority'].isna()), 'Seniority'] = 'Specialist'
    df.loc[(df['Title'].str.contains('Associate', 'Title')) &
           (df['Seniority'].isna()), 'Seniority'] = 'Associate'
    df['Seniority'] = df['Seniority'].fillna('other')

    return df


def filter_countries(df, countries):
    logging.info(f'Filtering the following countries: {countries}')
    df = df[df['Country'].isin(countries)]
    return df


def convert_kwkey_specialties_and_technologies_to_int(df):
    """Using regular expressions, converts a comma separated list of keys and technologies into the number of elements of this columns

    Args:
        df ([dataframe]): [Dataframe with the leads]

    Returns:
       df [dataframe]: [Dataframe with the leads]
    """
    logging.info(f'Counting KeyWords')
    # number of elements in kw_key
    df['Nkw_key'] = df.kw_key.apply(
        lambda x: 0 if pd.isnull(x) else len(re.findall("(?:\".*?\"|\S)+", x)))
    # Number of elements in technologies
    logging.info( f'Counting Technologies' )
    df['NTechnologies'] = df.Technologies.apply(
        lambda x: 0 if pd.isnull(x) else len(re.split(',', x)))
    logging.info( f'Counting Specialties' )
    df['NSpecialties'] = df.Technologies.apply(
        lambda x: 0 if pd.isnull(x) else len(re.split(',', x)))

    return df


def fill_historic_variables(df):
    """SQL window functions returns NULLS when there is no data, we reset this to 0
    Args:
        df ([dataframe]): [Dataframe with the leads]

    Returns:
        df [dataframe]: [Dataframe with the leads]
    """
    logging.info(
        "Filling historic variables ( PContacts, PStatus,Daysince last and first contact)")
    if 'PreviousContacts' in df:
        df['PreviousContacts'] = df['PreviousContacts'].fillna(0)
    if 'PreviousStatus' in df:
        df['PreviousStatus'] = df['PreviousStatus'].fillna(0)
        df['PreviousStatus'] = df['PreviousStatus'].astype("category")
    if 'DaysSinceLastContact' in df:
        df['DaysSinceLastContact'] = df['DaysSinceLastContact'].fillna(0)
    if 'DaysSinceFirstContact' in df:
        df['DaysSinceFirstContact'] = df['DaysSinceFirstContact'].fillna(0)

    return df


def impute_nas_and_save_imputer(df,columns_to_impute):
    df_impute = df[columns_to_impute]
    logging.info(f"Imputing NAs in {columns_to_impute}")
    logging.info(f"NAs in DataFrame before imputation:")
    nna = df_impute.isna().sum()
    logging.info(f"{nna}")

    logging.info("No imputer provided, instancing and fitting a new one")
    imp = IterativeImputer(missing_values=np.nan,
                            sample_posterior=False,
                            max_iter=10,
                            tol=0.01,
                            initial_strategy='mean',
                            estimator=BayesianRidge())

    imp.fit(df_impute)
    logging.info("Creating new imputer")
    df_impute = pd.DataFrame(data=imp.transform(df_impute),
                             columns=df_impute.columns)

    logging.info(f"NAs in DataFrame after imputation:")
    nna = df_impute.isna().sum()
    logging.info(f"{nna}")

    # Assign column
    for column in columns_to_impute:
        df[column] = df_impute[column].to_numpy()
    del df_impute

    return df,imp


def impute_nas(df,columns_to_impute,imp):

    """Imputes NAs in fields of interest, using sklearn's Iterative imputer

    Args:
        df ([dataframe]): [Dataframe with the leads]
        columns_to_impute ([list of columns]): [List of strings with the columns to impute]

    Returns:
        df [dataframe]: [Dataframe with the leads]
    """
    df_impute = df[columns_to_impute]
    logging.info(f"Imputing NAs in {columns_to_impute}")
    logging.info(f"NAs in DataFrame before imputation:")
    nna = df_impute.isna().sum()
    logging.info(f"{nna}")
    # Instance the imputer if it is not created

    print("Using provided imputer")
    df_impute = pd.DataFrame(data=imp.transform(df_impute),
                             columns=df_impute.columns)

    logging.info(f"NAs in DataFrame after imputation:")
    nna = df_impute.isna().sum()
    logging.info(f"{nna}")

    # Assign column
    for column in columns_to_impute:
        df[column] = df_impute[column].to_numpy()
    del df_impute

    return df


def log_transformation(df, columns):
    """Transform selected variables using log, imputing mean in negative values to avoid errors

    Args:
        df ([dataframe]): [datafram with the leads]
        columns ([list of strings]): [list of the columns to transform]
    Returns:
        df ([dataframe]): [datafram with the leads]
    """
    for column in columns:
        logging.info(f'Log Transforming : {column}')
        df.loc[(df[column] < 0), column] = df[column].mean()
        df[column] = df[column].apply(lambda x: np.log10(x + 1))
    return df


def create_month_status(df):
    logging.info("Creating Month of the date column")
    if 'campaign_bulk_date' in df.columns:

        df['MonthStatus'] = pd.DatetimeIndex(
            df['campaign_bulk_date']).month.astype('int16')
        df['MonthStatus'] = df['MonthStatus'].astype("category")
    return df


def clean_companyfollowers_text(df):
    """Deletes any word in the CompanyFollowers field, and converts to float

    Args:
        df ([dataframe]): dataframe with the leads

    Returns:
        df[dataframe]: [dataframe with the leads]
    """

    df['CompanyFollowers'] = df['CompanyFollowers'].str.extract(
        '(\d+)', expand=False).astype('float')
    return df


def clean_mxtype(df,mxtypes):
    """

    :param df: Input Dataset
    :param mxtypes: MXtypes to maintain ( the rest will go to others)
    :return df: processed Dataset
    """

    if 'MXtype' not in df.columns:
        raise Exception ("MXtype column not preset ( Check Caps)")
    logging.info('Defining MXTypes')
    logging.info(f'legal MXTypes: {mxtypes}')
    df.loc[~df['MXtype'].isin(mxtypes), 'MXtype'] = 'Others'
    logging.info(f'{df.MXtype.value_counts()}')
    return df


def clean_mxtype_using_dictionary(df,mx_bv_dict,mx_prob_dict):
    """

    :param df: Input Dataset
    :param mx_bv_dict: MXtypes dictionary by bv performance
    :param mx_prob_dict: MXtypes dictionary by prob performance
    :return df: processed Dataset
    """

    if 'MXtype' not in df.columns:
        raise Exception ("MXtype column not preset ( Check Caps)")
    logging.info('Defining MXTypes')
    df['mx_bv'] = df['MXtype']
    df = df.replace({'mx_bv': mx_bv_dict})
    logging.info(f'{df.mx_bv.value_counts()}')
    df['mx_prob'] = df['MXtype']
    df = df.replace( {'mx_prob': mx_prob_dict} )
    logging.info(f'{df.mx_prob.value_counts()}')
    return df





__all__ = ['save_df_for_s3']

def save_df_for_s3(df,path,dataset_type,keep_header=False,export_file_type='csv'):

    logging.info( f'Saving {dataset_type} features to {path} in format {export_file_type}')
    if 'idCampaign' in df:
        df.drop( ['idCampaign'], axis=1, inplace=True )

    logging.info( f'{dataset_type} data shape after preprocessing: {df.shape}' )
    if export_file_type =='zip':
        df.to_pickle(path=path,)
    elif export_file_type=='csv':
        df.to_csv( path, header=keep_header, index=False )
    else:
        raise Exception('Wrong export file type')
    logging.info("Complete")





import pandas as pd
import logging

__all__ = ['reduce_mem_usage','reduce_mem_usage_category']

import numpy as np
import pandas as pd

def reduce_mem_usage(df):
    """Reduces Memory usage of a DataFrame, converts strings to "object"

    Args:
        df ([Dataframe]): [input dataframe]

    Returns:
        df ([Dataframe]): [output dataframe]
    """
    start_mem = df.memory_usage().sum() / 1024**2
    logging.info('Memory usage of dataframe is {:.2f} MB'.format(start_mem))

    for col in df.columns:
        col_type = df[col].dtype

        if col_type != object:
            c_min = df[col].min()
            c_max = df[col].max()
            if str(col_type)[:3] == 'int':
                if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                    df[col] = df[col].astype(np.int8)
                elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                    df[col] = df[col].astype(np.int16)
                elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                    df[col] = df[col].astype(np.int32)
                elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                    df[col] = df[col].astype(np.int64)
            else:
                if c_min > np.finfo(np.float16).min and c_max < np.finfo(np.float16).max:
                    df[col] = df[col].astype(np.float16)
                elif c_min > np.finfo(np.float32).min and c_max < np.finfo(np.float32).max:
                    df[col] = df[col].astype(np.float32)
                else:
                    df[col] = df[col].astype(np.float64)
        else:
            df[col] = df[col].astype('object')

    end_mem = df.memory_usage().sum() / 1024**2
    logging.info(
        'Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    logging.info('Decreased by {:.1f}%'.format(
        100 * (start_mem - end_mem) / start_mem))

    return df


def reduce_mem_usage_category(df):
    """Reduces Memory usage of a DataFrame, converts string columns to Category, which needs less memory)"

    Args:
        df ([Dataframe]): [input dataframe]

    Returns:
        df ([Dataframe]): [output dataframe]
    """
    start_mem = df.memory_usage().sum() / 1024**2
    logging.info('Memory usage of dataframe is {:.2f} MB'.format(start_mem))

    for col in df.columns:
        col_type = df[col].dtype

        if col_type == object:
            df[col] = df[col].astype('category')

    end_mem = df.memory_usage().sum() / 1024**2
    logging.info(
        'Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    logging.info('Decreased by {:.1f}%'.format(
        100 * (start_mem - end_mem) / start_mem))
    return df


import pathlib
from setuptools import setup,find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="pyresdev",
    version="0.0.4",
    description="Bairesdev package used in DS",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Patricio Massaro",
    author_email="patricio.massaro@bairesdev.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["numpy", "pandas", "regex"],
)
